@@ -143,18 +143,16 @@ valid_ip (int family, const char *ip, GError **error)
 }
 
 static gboolean
-valid_prefix (int family, guint prefix, GError **error, gboolean allow_zero_prefix)
+valid_prefix (int family, guint prefix, GError **error)
 {
-	if (   (family == AF_INET && prefix > 32)
-	    || (family == AF_INET6 && prefix > 128)
-	    || (!allow_zero_prefix && prefix == 0)) {
-		g_set_error (error, NM_CONNECTION_ERROR, NM_CONNECTION_ERROR_FAILED,
-		             family == AF_INET ? _("Invalid IPv4 address prefix '%u'") : _("Invalid IPv6 address prefix '%u'"),
-		             prefix);
-		return FALSE;
-	}
+	if (   (family == AF_INET  && prefix <= 32)
+	    || (family == AF_INET6 && prefix <= 128))
+		return TRUE;
 
-	return TRUE;
+	g_set_error (error, NM_CONNECTION_ERROR, NM_CONNECTION_ERROR_FAILED,
+	             family == AF_INET ? _("Invalid IPv4 address prefix '%u'") : _("Invalid IPv6 address prefix '%u'"),
+	             prefix);
+	return FALSE;
 }
 
 static gboolean
@@ -175,6 +173,9 @@ valid_metric (gint64 metric, GError **error)
 	return TRUE;
 }
 
+/*****************************************************************************
+ * NMIPAddress
+ *****************************************************************************/
 
 G_DEFINE_BOXED_TYPE (NMIPAddress, nm_ip_address, nm_ip_address_dup, nm_ip_address_unref)
 
@@ -211,7 +212,7 @@ nm_ip_address_new (int family,
 
 	if (!valid_ip (family, addr, error))
 		return NULL;
-	if (!valid_prefix (family, prefix, error, FALSE))
+	if (!valid_prefix (family, prefix, error))
 		return NULL;
 
 	address = g_slice_new0 (NMIPAddress);
@@ -248,7 +249,7 @@ nm_ip_address_new_binary (int family,
 	g_return_val_if_fail (family == AF_INET || family == AF_INET6, NULL);
 	g_return_val_if_fail (addr != NULL, NULL);
 
-	if (!valid_prefix (family, prefix, error, FALSE))
+	if (!valid_prefix (family, prefix, error))
 		return NULL;
 
 	address = g_slice_new0 (NMIPAddress);
@@ -484,7 +485,7 @@ nm_ip_address_set_prefix (NMIPAddress *address,
                           guint prefix)
 {
 	g_return_if_fail (address != NULL);
-	g_return_if_fail (valid_prefix (address->family, prefix, NULL, FALSE));
+	g_return_if_fail (valid_prefix (address->family, prefix, NULL));
 
 	address->prefix = prefix;
 }
@@ -566,6 +567,9 @@ nm_ip_address_set_attribute (NMIPAddress *address, const char *name, GVariant *v
 		g_hash_table_remove (address->attributes, name);
 }
 
+/*****************************************************************************
+ * NMIPRoute
+ *****************************************************************************/
 
 G_DEFINE_BOXED_TYPE (NMIPRoute, nm_ip_route, nm_ip_route_dup, nm_ip_route_unref)
 
@@ -610,7 +614,7 @@ nm_ip_route_new (int family,
 
 	if (!valid_ip (family, dest, error))
 		return NULL;
-	if (!valid_prefix (family, prefix, error, TRUE))
+	if (!valid_prefix (family, prefix, error))
 		return NULL;
 	if (next_hop && !valid_ip (family, next_hop, error))
 		return NULL;
@@ -657,7 +661,7 @@ nm_ip_route_new_binary (int family,
 	g_return_val_if_fail (family == AF_INET || family == AF_INET6, NULL);
 	g_return_val_if_fail (dest, NULL);
 
-	if (!valid_prefix (family, prefix, error, TRUE))
+	if (!valid_prefix (family, prefix, error))
 		return NULL;
 	if (!valid_metric (metric, error))
 		return NULL;
@@ -898,7 +902,7 @@ nm_ip_route_set_prefix (NMIPRoute *route,
                         guint prefix)
 {
 	g_return_if_fail (route != NULL);
-	g_return_if_fail (valid_prefix (route->family, prefix, NULL, TRUE));
+	g_return_if_fail (valid_prefix (route->family, prefix, NULL));
 
 	route->prefix = prefix;
 }
@@ -1105,6 +1109,7 @@ nm_ip_route_set_attribute (NMIPRoute *route, const char *name, GVariant *value)
 		g_hash_table_remove (route->attributes, name);
 }
 
+/*****************************************************************************/
 
 G_DEFINE_ABSTRACT_TYPE (NMSettingIPConfig, nm_setting_ip_config, NM_TYPE_SETTING)
 
@@ -1115,6 +1120,7 @@ typedef struct {
 	GPtrArray *dns;        /* array of IP address strings */
 	GPtrArray *dns_search; /* array of domain name strings */
 	GPtrArray *dns_options;/* array of DNS options */
+	gint dns_priority;
 	GPtrArray *addresses;  /* array of NMIPAddress */
 	GPtrArray *routes;     /* array of NMIPRoute */
 	gint64 route_metric;
@@ -1135,6 +1141,7 @@ enum {
 	PROP_DNS,
 	PROP_DNS_SEARCH,
 	PROP_DNS_OPTIONS,
+	PROP_DNS_PRIORITY,
 	PROP_ADDRESSES,
 	PROP_GATEWAY,
 	PROP_ROUTES,
@@ -1678,6 +1685,22 @@ nm_setting_ip_config_clear_dns_options (NMSettingIPConfig *setting, gboolean is_
 	g_object_notify (G_OBJECT (setting), NM_SETTING_IP_CONFIG_DNS_OPTIONS);
 }
 
+/**
+ * nm_setting_ip_config_get_dns_priority:
+ * @setting: the #NMSettingIPConfig
+ *
+ * Returns: the priority of DNS servers
+ *
+ * Since: 1.2.4
+ **/
+gint
+nm_setting_ip_config_get_dns_priority (NMSettingIPConfig *setting)
+{
+	g_return_val_if_fail (NM_IS_SETTING_IP_CONFIG (setting), 0);
+
+	return NM_SETTING_IP_CONFIG_GET_PRIVATE (setting)->dns_priority;
+}
+
 /**
  * nm_setting_ip_config_get_num_addresses:
  * @setting: the #NMSettingIPConfig
@@ -2269,6 +2292,16 @@ verify (NMSetting *setting, NMConnection *connection, GError **error)
 		}
 	}
 
+	if (priv->gateway && priv->never_default) {
+		g_set_error (error,
+		             NM_CONNECTION_ERROR,
+		             NM_CONNECTION_ERROR_INVALID_PROPERTY,
+		             _("a gateway is incompatible with '%s'"),
+		             NM_SETTING_IP_CONFIG_NEVER_DEFAULT);
+		g_prefix_error (error, "%s.%s: ", nm_setting_get_name (setting), NM_SETTING_IP_CONFIG_GATEWAY);
+		return NM_SETTING_VERIFY_NORMALIZABLE_ERROR;
+	}
+
 	return TRUE;
 }
 
@@ -2347,6 +2380,9 @@ set_property (GObject *object, guint prop_id,
 			}
 		}
 		break;
+	case PROP_DNS_PRIORITY:
+		priv->dns_priority = g_value_get_int (value);
+		break;
 	case PROP_ADDRESSES:
 		g_ptr_array_unref (priv->addresses);
 		priv->addresses = _nm_utils_copy_array (g_value_get_boxed (value),
@@ -2419,6 +2455,9 @@ get_property (GObject *object, guint prop_id,
 	case PROP_DNS_OPTIONS:
 		g_value_take_boxed (value, priv->dns_options ? _nm_utils_ptrarray_to_strv (priv->dns_options) : NULL);
 		break;
+	case PROP_DNS_PRIORITY:
+		g_value_set_int (value, priv->dns_priority);
+		break;
 	case PROP_ADDRESSES:
 		g_value_take_boxed (value, _nm_utils_copy_array (priv->addresses,
 		                                                 (NMUtilsCopyFunc) nm_ip_address_dup,
@@ -2570,6 +2609,34 @@ nm_setting_ip_config_class_init (NMSettingIPConfigClass *setting_class)
 		                     G_PARAM_READWRITE |
 		                     G_PARAM_STATIC_STRINGS));
 
+	/**
+	 * NMSettingIPConfig:dns-priority:
+	 *
+	 * DNS priority.
+	 *
+	 * The relative priority to be used when determining the order of DNS
+	 * servers in resolv.conf.  A lower value means that servers will be on top
+	 * of the file.  Zero selects the default value, which is 50 for VPNs and
+	 * 100 for other connections.  When multiple devices have configurations
+	 * with the same priority, the one with an active default route will be
+	 * preferred.  Note that when using dns=dnsmasq the order is meaningless
+	 * since dnsmasq forwards queries to all known servers at the same time.
+	 *
+	 * Negative values have the special effect of excluding other configurations
+	 * with a greater priority value; so in presence of at least a negative
+	 * priority, only DNS servers from configurations with the lowest priority
+	 * value will be used.
+	 *
+	 * Since: 1.2.4
+	 **/
+	g_object_class_install_property
+	    (object_class, PROP_DNS_PRIORITY,
+	     g_param_spec_int (NM_SETTING_IP_CONFIG_DNS_PRIORITY, "", "",
+	                       G_MININT32, G_MAXINT32, 0,
+	                       G_PARAM_READWRITE |
+	                       G_PARAM_CONSTRUCT |
+	                       G_PARAM_STATIC_STRINGS));
+
 	/**
 	 * NMSettingIPConfig:addresses:
 	 *
