@@ -201,6 +201,18 @@ verify (NMSetting *setting, NMConnection *connection, GError **error)
 		return FALSE;
 	}
 
+	/* Failures from here on are NORMALIZABLE... */
+
+	if (   !strcmp (method, NM_SETTING_IP6_CONFIG_METHOD_IGNORE)
+	    && !nm_setting_ip_config_get_may_fail (s_ip)) {
+		g_set_error_literal (error,
+		                     NM_CONNECTION_ERROR,
+		                     NM_CONNECTION_ERROR_INVALID_PROPERTY,
+		                     _("property should be TRUE when method is set to ignore"));
+		g_prefix_error (error, "%s.%s: ", NM_SETTING_IP6_CONFIG_SETTING_NAME, NM_SETTING_IP_CONFIG_MAY_FAIL);
+		return NM_SETTING_VERIFY_NORMALIZABLE;
+	}
+
 	return TRUE;
 }
 
@@ -601,7 +613,7 @@ nm_setting_ip6_config_class_init (NMSettingIP6ConfigClass *ip6_class)
 	 *
 	 * Configure method for creating the address for use with RFC4862 IPv6
 	 * Stateless Address Autoconfiguration. The permitted values are: "eui64",
-	 * "stable-privacy" or unset.
+	 * or "stable-privacy".
 	 *
 	 * If the property is set to "eui64", the addresses will be generated
 	 * using the interface tokens derived from  hardware address. This makes
@@ -616,8 +628,10 @@ nm_setting_ip6_config_class_init (NMSettingIP6ConfigClass *ip6_class)
 	 * and makes the address stable when the network interface hardware is
 	 * replaced.
 	 *
-	 * Leaving this unset causes a default that could be subject to change
-	 * in future versions to be used.
+	 * On D-Bus, the absence of an addr-gen-mode setting equals enabling
+	 * "stable-privacy". For keyfile plugin, the absence of the setting
+	 * on disk means "eui64" so that the property doesn't change on upgrade
+	 * from older versions.
 	 *
 	 * Note that this setting is distinct from the Privacy Extensions as
 	 * configured by "ip6-privacy" property and it does not affect the
