@@ -1,7 +1,5 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /*
- * libnm_glib -- Access network status & information from glib applications
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -17,8 +15,8 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301 USA.
  *
- * Copyright (C) 2011 - 2012 Red Hat, Inc.
- * Copyright (C) 2009 Novell, Inc.
+ * Copyright 2011 - 2012 Red Hat, Inc.
+ * Copyright 2009 Novell, Inc.
  */
 
 #ifndef NM_DEVICE_WIMAX_H
@@ -53,6 +51,7 @@ typedef enum {
 } NMDeviceWimaxError;
 
 #define NM_DEVICE_WIMAX_ERROR nm_device_wimax_error_quark ()
+NM_DEPRECATED_IN_1_2
 GQuark nm_device_wimax_error_quark (void);
 
 #define NM_DEVICE_WIMAX_HW_ADDRESS       "hw-address"
@@ -76,22 +75,33 @@ typedef struct {
 	void (*nsp_removed) (NMDeviceWimax *self, NMWimaxNsp *nsp);
 } NMDeviceWimaxClass;
 
+NM_DEPRECATED_IN_1_2
 GType nm_device_wimax_get_type (void);
 
+NM_DEPRECATED_IN_1_2
 GObject         *nm_device_wimax_new             (DBusGConnection *connection,
-												  const char *path);
+                                                  const char *path);
 
+NM_DEPRECATED_IN_1_2
 const char      *nm_device_wimax_get_hw_address  (NMDeviceWimax *wimax);
+NM_DEPRECATED_IN_1_2
 NMWimaxNsp      *nm_device_wimax_get_active_nsp  (NMDeviceWimax *wimax);
+NM_DEPRECATED_IN_1_2
 NMWimaxNsp      *nm_device_wimax_get_nsp_by_path (NMDeviceWimax *wimax,
-												  const char *path);
+                                                  const char *path);
 
+NM_DEPRECATED_IN_1_2
 const GPtrArray *nm_device_wimax_get_nsps        (NMDeviceWimax *wimax);
 
+NM_DEPRECATED_IN_1_2
 guint            nm_device_wimax_get_center_frequency (NMDeviceWimax *self);
+NM_DEPRECATED_IN_1_2
 gint             nm_device_wimax_get_rssi        (NMDeviceWimax *self);
+NM_DEPRECATED_IN_1_2
 gint             nm_device_wimax_get_cinr        (NMDeviceWimax *self);
+NM_DEPRECATED_IN_1_2
 gint             nm_device_wimax_get_tx_power    (NMDeviceWimax *self);
+NM_DEPRECATED_IN_1_2
 const char *     nm_device_wimax_get_bsid        (NMDeviceWimax *self);
 
 G_END_DECLS
