@@ -1,7 +1,5 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /*
- * libnm_glib -- Access network status & information from glib applications
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -17,10 +15,12 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301 USA.
  *
- * Copyright (C) 2008 - 2011 Red Hat, Inc.
- * Copyright (C) 2008 Novell, Inc.
+ * Copyright 2008 - 2011 Red Hat, Inc.
+ * Copyright 2008 Novell, Inc.
  */
 
+#include "nm-default.h"
+
 #include <string.h>
 
 #include "nm-dhcp4-config.h"
@@ -51,26 +51,23 @@ nm_dhcp4_config_init (NMDHCP4Config *config)
 {
 }
 
-static void
-copy_options (gpointer key, gpointer data, gpointer user_data)
-{
-	GHashTable *options = (GHashTable *) user_data;
-	GValue *value = (GValue *) data;
-
-	g_hash_table_insert (options, g_strdup (key), g_value_dup_string (value));
-}
-
 static gboolean
 demarshal_dhcp4_options (NMObject *object, GParamSpec *pspec, GValue *value, gpointer field)
 {
 	NMDHCP4ConfigPrivate *priv = NM_DHCP4_CONFIG_GET_PRIVATE (object);
 	GHashTable *new_options;
+	GHashTableIter iter;
+	const char *key;
+	GValue *opt;
 
 	g_hash_table_remove_all (priv->options);
 
 	new_options = g_value_get_boxed (value);
-	if (new_options)
-		g_hash_table_foreach (new_options, copy_options, priv->options);
+	if (new_options) {
+		g_hash_table_iter_init (&iter, new_options);
+		while (g_hash_table_iter_next (&iter, (gpointer) &key, (gpointer) &opt))
+			g_hash_table_insert (priv->options, g_strdup (key), g_value_dup_string (opt));
+	}
 
 	_nm_object_queue_notify (object, NM_DHCP4_CONFIG_OPTIONS);
 	return TRUE;
@@ -93,21 +90,13 @@ register_properties (NMDHCP4Config *config)
 static void
 constructed (GObject *object)
 {
-	DBusGConnection *connection;
-	NMDHCP4ConfigPrivate *priv;
+	NMDHCP4ConfigPrivate *priv = NM_DHCP4_CONFIG_GET_PRIVATE (object);
 
 	G_OBJECT_CLASS (nm_dhcp4_config_parent_class)->constructed (object);
 
-	priv = NM_DHCP4_CONFIG_GET_PRIVATE (object);
 	priv->options = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
-	connection = nm_object_get_connection (NM_OBJECT (object));
-
-	priv->proxy = dbus_g_proxy_new_for_name (connection,
-										   NM_DBUS_SERVICE,
-										   nm_object_get_path (NM_OBJECT (object)),
-										   NM_DBUS_INTERFACE_DHCP4_CONFIG);
-
+	priv->proxy = _nm_object_new_proxy (NM_OBJECT (object), NULL, NM_DBUS_INTERFACE_DHCP4_CONFIG);
 	register_properties (NM_DHCP4_CONFIG (object));
 }
 
@@ -167,11 +156,10 @@ nm_dhcp4_config_class_init (NMDHCP4ConfigClass *config_class)
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_OPTIONS,
-		 g_param_spec_boxed (NM_DHCP4_CONFIG_OPTIONS,
-						       "Options",
-						       "Options",
-						       G_TYPE_HASH_TABLE,
-						       G_PARAM_READABLE));
+		 g_param_spec_boxed (NM_DHCP4_CONFIG_OPTIONS, "", "",
+		                     G_TYPE_HASH_TABLE,
+		                     G_PARAM_READABLE |
+		                     G_PARAM_STATIC_STRINGS));
 }
 
 /**
@@ -187,9 +175,9 @@ GObject *
 nm_dhcp4_config_new (DBusGConnection *connection, const char *object_path)
 {
 	return (GObject *) g_object_new (NM_TYPE_DHCP4_CONFIG,
-									 NM_OBJECT_DBUS_CONNECTION, connection,
-									 NM_OBJECT_DBUS_PATH, object_path,
-									 NULL);
+	                                 NM_OBJECT_DBUS_CONNECTION, connection,
+	                                 NM_OBJECT_DBUS_PATH, object_path,
+	                                 NULL);
 }
 
 /**
@@ -227,4 +215,3 @@ nm_dhcp4_config_get_one_option (NMDHCP4Config *config, const char *option)
 
 	return g_hash_table_lookup (nm_dhcp4_config_get_options (config), option);
 }
-
