@@ -15,18 +15,19 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2010 - 2011 Red Hat, Inc.
+ * Copyright 2010 - 2011 Red Hat, Inc.
  */
 
-#include <config.h>
+#include "nm-default.h"
+
 #include <string.h>
 #include <dbus/dbus-glib-lowlevel.h>
 
-#include "nm-glib-compat.h"
 #include "NetworkManager.h"
 #include "nm-secret-agent.h"
 #include "nm-glib-enum-types.h"
 #include "nm-dbus-helpers-private.h"
+#include "nm-setting-private.h"
 
 static void impl_secret_agent_get_secrets (NMSecretAgent *self,
                                            GHashTable *connection_hash,
@@ -55,9 +56,7 @@ static void impl_secret_agent_delete_secrets (NMSecretAgent *self,
 
 G_DEFINE_ABSTRACT_TYPE (NMSecretAgent, nm_secret_agent, G_TYPE_OBJECT)
 
-#define NM_SECRET_AGENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), \
-                                        NM_TYPE_SECRET_AGENT, \
-                                        NMSecretAgentPrivate))
+#define NM_SECRET_AGENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_TYPE_SECRET_AGENT, NMSecretAgentPrivate))
 
 static gboolean auto_register_cb (gpointer user_data);
 
@@ -66,7 +65,6 @@ typedef struct {
 	NMSecretAgentCapabilities capabilities;
 
 	DBusGConnection *bus;
-	gboolean private_bus;
 	DBusGProxy *dbus_proxy;
 	DBusGProxy *manager_proxy;
 	DBusGProxyCall *reg_call;
@@ -124,11 +122,11 @@ get_nm_owner (NMSecretAgent *self)
 
 	if (!priv->nm_owner) {
 		if (!dbus_g_proxy_call_with_timeout (priv->dbus_proxy,
-			                                 "GetNameOwner", 2000, &error,
-			                                 G_TYPE_STRING, NM_DBUS_SERVICE,
-			                                 G_TYPE_INVALID,
-			                                 G_TYPE_STRING, &owner,
-			                                 G_TYPE_INVALID))
+		                                     "GetNameOwner", 2000, &error,
+		                                     G_TYPE_STRING, NM_DBUS_SERVICE,
+		                                     G_TYPE_INVALID,
+		                                     G_TYPE_STRING, &owner,
+		                                     G_TYPE_INVALID))
 			return NULL;
 
 		priv->nm_owner = g_strdup (owner);
@@ -228,12 +226,6 @@ verify_sender (NMSecretAgent *self,
 
 	g_return_val_if_fail (context != NULL, FALSE);
 
-	/* Private bus connection is always to NetworkManager, which is always
-	 * UID 0.
-	 */
-	if (priv->private_bus)
-		return TRUE;
-
 	/* Verify the sender's UID is 0, and that the sender is the same as
 	 * NetworkManager's bus name owner.
 	 */
@@ -311,7 +303,8 @@ verify_request (NMSecretAgent *self,
                 GError **error)
 {
 	NMConnection *connection = NULL;
-	GError *local = NULL;
+
+	g_return_val_if_fail (out_connection, FALSE);
 
 	if (!verify_sender (self, context, error))
 		return FALSE;
@@ -330,22 +323,11 @@ verify_request (NMSecretAgent *self,
 	}
 
 	/* Make sure the given connection is valid */
-	g_assert (out_connection);
-	connection = nm_connection_new_from_hash (connection_hash, &local);
-	if (connection) {
-		nm_connection_set_path (connection, connection_path);
-		*out_connection = connection;
-	} else {
-		g_set_error (error,
-		             NM_SECRET_AGENT_ERROR,
-		             NM_SECRET_AGENT_ERROR_INVALID_CONNECTION,
-		             "Invalid connection: (%d) %s",
-		             local ? local->code : -1,
-		             (local && local->message) ? local->message : "(unknown)");
-		g_clear_error (&local);
-	}
+	connection = _nm_connection_new_from_hash (connection_hash);
+	nm_connection_set_path (connection, connection_path);
+	*out_connection = connection;
 
-	return !!connection;
+	return TRUE;
 }
 
 static void
@@ -627,7 +609,7 @@ nm_secret_agent_register (NMSecretAgent *self)
 	g_return_val_if_fail (class->save_secrets != NULL, FALSE);
 	g_return_val_if_fail (class->delete_secrets != NULL, FALSE);
 
-	if (!priv->nm_owner && !priv->private_bus)
+	if (!priv->nm_owner)
 		return FALSE;
 
 	priv->suppress_auto = FALSE;
@@ -672,7 +654,7 @@ nm_secret_agent_unregister (NMSecretAgent *self)
 	g_return_val_if_fail (priv->bus != NULL, FALSE);
 	g_return_val_if_fail (priv->manager_proxy != NULL, FALSE);
 
-	if (!priv->nm_owner && !priv->private_bus)
+	if (!priv->nm_owner)
 		return FALSE;
 
 	dbus_g_proxy_call_no_reply (priv->manager_proxy, "Unregister", G_TYPE_INVALID);
@@ -704,7 +686,8 @@ auto_register_cb (gpointer user_data)
 	NMSecretAgentPrivate *priv = NM_SECRET_AGENT_GET_PRIVATE (self);
 
 	priv->auto_register_id = 0;
-	if (priv->auto_register && !priv->suppress_auto && (priv->reg_call == NULL))
+	if (priv->auto_register && !priv->suppress_auto &&
+	   (priv->reg_call == NULL && !priv->registered))
 		nm_secret_agent_register (self);
 	return FALSE;
 }
@@ -853,29 +836,26 @@ nm_secret_agent_init (NMSecretAgent *self)
 		g_error_free (error);
 		return;
 	}
-	priv->private_bus = _nm_dbus_is_connection_private (priv->bus);
 
-	if (priv->private_bus == FALSE) {
-		priv->dbus_proxy = dbus_g_proxy_new_for_name (priv->bus,
-		                                              DBUS_SERVICE_DBUS,
-		                                              DBUS_PATH_DBUS,
-		                                              DBUS_INTERFACE_DBUS);
-		g_assert (priv->dbus_proxy);
+	priv->dbus_proxy = dbus_g_proxy_new_for_name (priv->bus,
+	                                              DBUS_SERVICE_DBUS,
+	                                              DBUS_PATH_DBUS,
+	                                              DBUS_INTERFACE_DBUS);
+	g_assert (priv->dbus_proxy);
 
-		dbus_g_object_register_marshaller (g_cclosure_marshal_generic,
-		                                   G_TYPE_NONE,
-		                                   G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
-		                                   G_TYPE_INVALID);
-		dbus_g_proxy_add_signal (priv->dbus_proxy, "NameOwnerChanged",
-		                         G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
-		                         G_TYPE_INVALID);
-		dbus_g_proxy_connect_signal (priv->dbus_proxy,
-		                             "NameOwnerChanged",
-		                             G_CALLBACK (name_owner_changed),
-		                             self, NULL);
+	dbus_g_object_register_marshaller (g_cclosure_marshal_generic,
+	                                   G_TYPE_NONE,
+	                                   G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
+	                                   G_TYPE_INVALID);
+	dbus_g_proxy_add_signal (priv->dbus_proxy, "NameOwnerChanged",
+	                         G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
+	                         G_TYPE_INVALID);
+	dbus_g_proxy_connect_signal (priv->dbus_proxy,
+	                             "NameOwnerChanged",
+	                             G_CALLBACK (name_owner_changed),
+	                             self, NULL);
 
-		get_nm_owner (self);
-	}
+	get_nm_owner (self);
 
 	priv->manager_proxy = _nm_dbus_new_proxy_for_connection (priv->bus,
 	                                                         NM_DBUS_PATH_AGENT_MANAGER,
@@ -885,7 +865,7 @@ nm_secret_agent_init (NMSecretAgent *self)
 		return;
 	}
 
-	if (priv->nm_owner || priv->private_bus)
+	if (priv->nm_owner)
 		priv->auto_register_id = g_idle_add (auto_register_cb, self);
 }
 
@@ -1004,11 +984,11 @@ nm_secret_agent_class_init (NMSecretAgentClass *class)
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_IDENTIFIER,
-		 g_param_spec_string (NM_SECRET_AGENT_IDENTIFIER,
-						      "Identifier",
-						      "Identifier",
-						      NULL,
-						      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+		 g_param_spec_string (NM_SECRET_AGENT_IDENTIFIER, "", "",
+		                      NULL,
+		                      G_PARAM_READWRITE |
+		                      G_PARAM_CONSTRUCT_ONLY |
+		                      G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * NMSecretAgent:auto-register:
@@ -1023,11 +1003,11 @@ nm_secret_agent_class_init (NMSecretAgentClass *class)
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_AUTO_REGISTER,
-		 g_param_spec_boolean (NM_SECRET_AGENT_AUTO_REGISTER,
-						       "Auto Register",
-						       "Auto Register",
-						       TRUE,
-						       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+		 g_param_spec_boolean (NM_SECRET_AGENT_AUTO_REGISTER, "", "",
+		                       TRUE,
+		                       G_PARAM_READWRITE |
+		                       G_PARAM_CONSTRUCT |
+		                       G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * NMSecretAgent:registered:
@@ -1036,11 +1016,10 @@ nm_secret_agent_class_init (NMSecretAgentClass *class)
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_REGISTERED,
-		 g_param_spec_boolean (NM_SECRET_AGENT_REGISTERED,
-		                       "Registered",
-		                       "Registered",
+		 g_param_spec_boolean (NM_SECRET_AGENT_REGISTERED, "", "",
 		                       FALSE,
-		                       G_PARAM_READABLE));
+		                       G_PARAM_READABLE |
+		                       G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * NMSecretAgent:capabilities:
@@ -1049,12 +1028,12 @@ nm_secret_agent_class_init (NMSecretAgentClass *class)
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_CAPABILITIES,
-		 g_param_spec_flags (NM_SECRET_AGENT_CAPABILITIES,
-		                     "Capabilities",
-		                     "Capabilities",
+		 g_param_spec_flags (NM_SECRET_AGENT_CAPABILITIES, "", "",
 		                     NM_TYPE_SECRET_AGENT_CAPABILITIES,
 		                     NM_SECRET_AGENT_CAPABILITY_NONE,
-		                     G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+		                     G_PARAM_READWRITE |
+		                     G_PARAM_CONSTRUCT |
+		                     G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * NMSecretAgent::registration-result:
@@ -1066,10 +1045,10 @@ nm_secret_agent_class_init (NMSecretAgentClass *class)
 	 **/
 	signals[REGISTRATION_RESULT] =
 		g_signal_new (NM_SECRET_AGENT_REGISTRATION_RESULT,
-					  G_OBJECT_CLASS_TYPE (object_class),
-					  G_SIGNAL_RUN_FIRST,
-					  0, NULL, NULL, NULL,
-					  G_TYPE_NONE, 1, G_TYPE_POINTER);
+		              G_OBJECT_CLASS_TYPE (object_class),
+		              G_SIGNAL_RUN_FIRST,
+		              0, NULL, NULL, NULL,
+		              G_TYPE_NONE, 1, G_TYPE_POINTER);
 
 	dbus_g_object_type_install_info (G_TYPE_FROM_CLASS (class),
 	                                 &dbus_glib_nm_secret_agent_object_info);
@@ -1078,4 +1057,3 @@ nm_secret_agent_class_init (NMSecretAgentClass *class)
 	                              NM_DBUS_INTERFACE_SECRET_AGENT,
 	                              NM_TYPE_SECRET_AGENT_ERROR);
 }
-
