@@ -90,18 +90,4 @@
 # define NM_AVAILABLE_IN_1_2
 #endif
 
-#if NM_VERSION_MIN_REQUIRED >= NM_VERSION_1_4
-# define NM_DEPRECATED_IN_1_4           G_DEPRECATED
-# define NM_DEPRECATED_IN_1_4_FOR(f)    G_DEPRECATED_FOR(f)
-#else
-# define NM_DEPRECATED_IN_1_4
-# define NM_DEPRECATED_IN_1_4_FOR(f)
-#endif
-
-#if NM_VERSION_MAX_ALLOWED < NM_VERSION_1_4
-# define NM_AVAILABLE_IN_1_4            G_UNAVAILABLE(1,4)
-#else
-# define NM_AVAILABLE_IN_1_4
-#endif
-
 #endif  /* NM_VERSION_H */
