@@ -179,6 +179,14 @@ typedef struct {
 	 */
 	void        (*realize_start_notify) (NMDevice *self, const NMPlatformLink *plink);
 
+	/**
+	 * unrealize():
+	 * @self: the #NMDevice
+	 *
+	 * Remove the device backing resources.
+	 */
+	gboolean               (*unrealize) (NMDevice *self, GError **error);
+
 	/**
 	 * unrealize_notify():
 	 * @self: the #NMDevice
@@ -377,7 +385,7 @@ gboolean        nm_device_has_unmodified_applied_connection (NMDevice *self,
                                                              NMSettingCompareFlags compare_flags);
 NMSetting *     nm_device_get_applied_setting   (NMDevice *dev, GType setting_type);
 
-void            nm_device_removed               (NMDevice *dev);
+void            nm_device_removed               (NMDevice *self, gboolean unconfigure_ip_config);
 
 gboolean        nm_device_is_available          (NMDevice *dev, NMDeviceCheckDevAvailableFlags flags);
 gboolean        nm_device_has_carrier           (NMDevice *dev);
@@ -481,6 +489,10 @@ void nm_device_set_unmanaged_by_flags (NMDevice *device,
                                        NMUnmanagedFlags flags,
                                        NMUnmanFlagOp set_op,
                                        NMDeviceStateReason reason);
+void nm_device_set_unmanaged_by_flags_queue (NMDevice *self,
+                                             NMUnmanagedFlags flags,
+                                             NMUnmanFlagOp set_op,
+                                             NMDeviceStateReason reason);
 void nm_device_set_unmanaged_by_user_config (NMDevice *self, const GSList *unmanaged_specs);
 void nm_device_set_unmanaged_by_user_udev (NMDevice *self);
 void nm_device_set_unmanaged_by_quitting (NMDevice *device);
