@@ -19,81 +19,73 @@
  * Copyright (C) 2006 - 2008 Novell, Inc.
  */
 
-#ifndef NM_DHCP_MANAGER_H
-#define NM_DHCP_MANAGER_H
+#ifndef __NETWORKMANAGER_DHCP_MANAGER_H__
+#define __NETWORKMANAGER_DHCP_MANAGER_H__
 
-#include <glib.h>
-#include <glib-object.h>
-
-#include <nm-setting-ip4-config.h>
-#include <nm-setting-ip6-config.h>
 
+#include "nm-default.h"
 #include "nm-dhcp-client.h"
 #include "nm-ip4-config.h"
 #include "nm-dhcp4-config.h"
 
-typedef enum {
-	NM_DHCP_MANAGER_ERROR_BAD_CLIENT = 0, /*< nick=BadClient >*/
-	NM_DHCP_MANAGER_ERROR_INTERNAL = 1,   /*< nick=InternalError >*/
-} NMDHCPManagerError;
-
-#define NM_DHCP_MANAGER_ERROR (nm_dhcp_manager_error_quark ())
-
-GQuark nm_dhcp_manager_error_quark    (void);
-
-
 #define NM_TYPE_DHCP_MANAGER            (nm_dhcp_manager_get_type ())
-#define NM_DHCP_MANAGER(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NM_TYPE_DHCP_MANAGER, NMDHCPManager))
-#define NM_DHCP_MANAGER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), NM_TYPE_DHCP_MANAGER, NMDHCPManagerClass))
+#define NM_DHCP_MANAGER(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NM_TYPE_DHCP_MANAGER, NMDhcpManager))
+#define NM_DHCP_MANAGER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), NM_TYPE_DHCP_MANAGER, NMDhcpManagerClass))
 #define NM_IS_DHCP_MANAGER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NM_TYPE_DHCP_MANAGER))
 #define NM_IS_DHCP_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), NM_TYPE_DHCP_MANAGER))
-#define NM_DHCP_MANAGER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), NM_TYPE_DHCP_MANAGER, NMDHCPManagerClass))
+#define NM_DHCP_MANAGER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), NM_TYPE_DHCP_MANAGER, NMDhcpManagerClass))
 
 typedef struct {
 	GObject parent;
-} NMDHCPManager;
+} NMDhcpManager;
 
 typedef struct {
 	GObjectClass parent;
-} NMDHCPManagerClass;
+} NMDhcpManagerClass;
 
 GType nm_dhcp_manager_get_type (void);
 
-NMDHCPManager *nm_dhcp_manager_get (void);
+NMDhcpManager *nm_dhcp_manager_get (void);
 
-void           nm_dhcp_manager_set_default_hostname (NMDHCPManager *manager,
+void           nm_dhcp_manager_set_default_hostname (NMDhcpManager *manager,
                                                      const char *hostname);
 
-NMDHCPClient * nm_dhcp_manager_start_ip4     (NMDHCPManager *manager,
+NMDhcpClient * nm_dhcp_manager_start_ip4     (NMDhcpManager *manager,
                                               const char *iface,
+                                              int ifindex,
                                               const GByteArray *hwaddr,
                                               const char *uuid,
-                                              guint priority,
-                                              NMSettingIP4Config *s_ip4,
+                                              guint32 priority,
+                                              gboolean send_hostname,
+                                              const char *dhcp_hostname,
+                                              const char *dhcp_fqdn,
+                                              const char *dhcp_client_id,
                                               guint32 timeout,
-                                              GByteArray *dhcp_anycast_addr);
+                                              const char *dhcp_anycast_addr,
+                                              const char *last_ip_address);
 
-NMDHCPClient * nm_dhcp_manager_start_ip6     (NMDHCPManager *manager,
+NMDhcpClient * nm_dhcp_manager_start_ip6     (NMDhcpManager *manager,
                                               const char *iface,
+                                              int ifindex,
                                               const GByteArray *hwaddr,
+                                              const struct in6_addr *ll_addr,
                                               const char *uuid,
-                                              guint priority,
-                                              NMSettingIP6Config *s_ip6,
+                                              guint32 priority,
+                                              gboolean send_hostname,
+                                              const char *dhcp_hostname,
                                               guint32 timeout,
-                                              GByteArray *dhcp_anycast_addr,
-                                              gboolean info_only);
+                                              const char *dhcp_anycast_addr,
+                                              gboolean info_only,
+                                              NMSettingIP6ConfigPrivacy privacy);
 
-GSList *       nm_dhcp_manager_get_lease_ip_configs (NMDHCPManager *self,
+GSList *       nm_dhcp_manager_get_lease_ip_configs (NMDhcpManager *self,
                                                      const char *iface,
+                                                     int ifindex,
                                                      const char *uuid,
-                                                     gboolean ipv6);
+                                                     gboolean ipv6,
+                                                     guint32 default_route_metric);
 
 /* For testing only */
-NMIP4Config *nm_dhcp_manager_test_ip4_options_to_config (const char *dhcp_client,
-                                                         const char *iface,
-                                                         GHashTable *options,
-                                                         const char *reason);
-
 extern const char* nm_dhcp_helper_path;
 
-#endif /* NM_DHCP_MANAGER_H */
+#endif /* __NETWORKMANAGER_DHCP_MANAGER_H__ */
