@@ -213,7 +213,8 @@ rdisc_config_changed (NMRDisc *rdisc, NMRDiscConfigMap changed, gpointer user_da
 			 * local configuration or user preferences are, so sending routes
 			 * with a prefix length of 0 is quite rude and thus ignored.
 			 */
-			if (discovered_route->plen > 0) {
+			if (   discovered_route->plen > 0
+			    && discovered_route->plen <= 128) {
 				memset (&route, 0, sizeof (route));
 				route.network = discovered_route->network;
 				route.plen = discovered_route->plen;
@@ -526,6 +527,8 @@ gboolean nm_config_get_configure_and_quit (gpointer unused);
 gconstpointer nm_bus_manager_get (void);
 void nm_bus_manager_register_object (gpointer unused, gpointer object);
 void nm_bus_manager_unregister_object (gpointer unused, gpointer object);
+GType nm_device_get_type (void);
+GType nm_active_connection_get_type (void);
 
 gconstpointer
 nm_config_get (void)
@@ -561,3 +564,15 @@ nm_bus_manager_unregister_object (gpointer unused, gpointer object)
 {
 }
 
+GType
+nm_device_get_type (void)
+{
+	g_return_val_if_reached (0);
+}
+
+GType
+nm_active_connection_get_type (void)
+{
+	g_return_val_if_reached (0);
+}
+
