@@ -54,4 +54,7 @@ const char *nmp_utils_udev_get_driver (GUdevDevice *device);
 
 gboolean nmp_utils_device_exists (const char *name);
 
+guint            nmp_utils_ip_config_source_to_rtprot   (NMIPConfigSource source);
+NMIPConfigSource nmp_utils_ip_config_source_from_rtprot (guint rtprot);
+
 #endif /* __NM_PLATFORM_UTILS_H__ */
