@@ -18,26 +18,11 @@
  * Copyright (C) 2010 - 2011 Red Hat, Inc.
  */
 
-#ifndef NM_SECRET_AGENT_H
-#define NM_SECRET_AGENT_H
-
-#include <glib.h>
-#include <glib-object.h>
-#include <dbus/dbus-glib.h>
-#include <dbus/dbus-glib-lowlevel.h>
+#ifndef __NETWORKMANAGER_SECRET_AGENT_H__
+#define __NETWORKMANAGER_SECRET_AGENT_H__
 
 #include <nm-connection.h>
-#include "nm-dbus-manager.h"
-#include "nm-settings-flags.h"
-#include "nm-auth-subject.h"
-
-/* NOTE: ensure these capabilities match those in introspection/nm-secret-agent.xml and
- * libnm-glib/nm-secret-agent.h.
- */
-typedef enum {
-	NM_SECRET_AGENT_CAPABILITY_NONE = 0x0,
-	NM_SECRET_AGENT_CAPABILITY_VPN_HINTS = 0x1,
-} NMSecretAgentCapabilities;
+#include "nm-default.h"
 
 #define NM_TYPE_SECRET_AGENT            (nm_secret_agent_get_type ())
 #define NM_SECRET_AGENT(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NM_TYPE_SECRET_AGENT, NMSecretAgent))
@@ -46,17 +31,23 @@ typedef enum {
 #define NM_IS_SECRET_AGENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), NM_TYPE_SECRET_AGENT))
 #define NM_SECRET_AGENT_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), NM_TYPE_SECRET_AGENT, NMSecretAgentClass))
 
-typedef struct {
+#define NM_SECRET_AGENT_DISCONNECTED "disconnected"
+
+struct _NMSecretAgent {
 	GObject parent;
-} NMSecretAgent;
+};
 
 typedef struct {
 	GObjectClass parent;
+
+	void (*disconnected) (NMSecretAgent *self);
 } NMSecretAgentClass;
 
+typedef struct _NMSecretAgentCallId *NMSecretAgentCallId;
+
 GType nm_secret_agent_get_type (void);
 
-NMSecretAgent *nm_secret_agent_new (DBusGMethodInvocation *context,
+NMSecretAgent *nm_secret_agent_new (GDBusMethodInvocation *context,
                                     NMAuthSubject *subject,
                                     const char *identifier,
                                     NMSecretAgentCapabilities capabilities);
@@ -75,8 +66,6 @@ gulong      nm_secret_agent_get_pid        (NMSecretAgent *agent);
 
 NMSecretAgentCapabilities nm_secret_agent_get_capabilities (NMSecretAgent *agent);
 
-guint32     nm_secret_agent_get_hash       (NMSecretAgent *agent);
-
 NMAuthSubject *nm_secret_agent_get_subject (NMSecretAgent *agent);
 
 void        nm_secret_agent_add_permission (NMSecretAgent *agent,
@@ -87,30 +76,33 @@ gboolean    nm_secret_agent_has_permission (NMSecretAgent *agent,
                                             const char *permission);
 
 typedef void (*NMSecretAgentCallback) (NMSecretAgent *agent,
-                                       gconstpointer call,
-                                       GHashTable *new_secrets, /* NULL for save & delete */
+                                       NMSecretAgentCallId call_id,
+                                       GVariant *new_secrets, /* NULL for save & delete */
                                        GError *error,
                                        gpointer user_data);
 
-gconstpointer nm_secret_agent_get_secrets  (NMSecretAgent *agent,
-                                            NMConnection *connection,
-                                            const char *setting_name,
-                                            const char **hints,
-                                            NMSettingsGetSecretsFlags flags,
-                                            NMSecretAgentCallback callback,
-                                            gpointer callback_data);
+NMSecretAgentCallId nm_secret_agent_get_secrets  (NMSecretAgent *agent,
+                                                  const char *path,
+                                                  NMConnection *connection,
+                                                  const char *setting_name,
+                                                  const char **hints,
+                                                  NMSecretAgentGetSecretsFlags flags,
+                                                  NMSecretAgentCallback callback,
+                                                  gpointer callback_data);
 
 void        nm_secret_agent_cancel_secrets (NMSecretAgent *agent,
-                                            gconstpointer call_id);
+                                            NMSecretAgentCallId call_id);
 
-gconstpointer nm_secret_agent_save_secrets (NMSecretAgent *agent,
-                                            NMConnection *connection,
-                                            NMSecretAgentCallback callback,
-                                            gpointer callback_data);
+NMSecretAgentCallId nm_secret_agent_save_secrets (NMSecretAgent *agent,
+                                                  const char *path,
+                                                  NMConnection *connection,
+                                                  NMSecretAgentCallback callback,
+                                                  gpointer callback_data);
 
-gconstpointer nm_secret_agent_delete_secrets (NMSecretAgent *agent,
-                                              NMConnection *connection,
-                                              NMSecretAgentCallback callback,
-                                              gpointer callback_data);
+NMSecretAgentCallId nm_secret_agent_delete_secrets (NMSecretAgent *agent,
+                                                    const char *path,
+                                                    NMConnection *connection,
+                                                    NMSecretAgentCallback callback,
+                                                    gpointer callback_data);
 
-#endif /* NM_SECRET_AGENT_H */
+#endif /* __NETWORKMANAGER_SECRET_AGENT_H__ */
