@@ -19,10 +19,12 @@
  * Copyright (C) 1999-2010 Gentoo Foundation, Inc.
  */
 
+#include "nm-default.h"
+
 #include <string.h>
 #include <stdlib.h>
-#include <nm-system-config-interface.h>
-#include <nm-logging.h>
+
+#include "nm-settings-plugin.h"
 #include "wpa_parser.h"
 #include "net_parser.h"
 #include "net_utils.h"
@@ -166,7 +168,7 @@ add_one_wep_key (GHashTable * table, int key_num, gchar * one_wep_key)
 /* Reading wep security information from /etc/conf.d/net.
  * This should not be used in future, use wpa_supplicant instead. */
 static void
-add_keys_from_net ()
+add_keys_from_net (void)
 {
 	GList *names = ifnet_get_connection_names ();
 	GList *iter = names;
@@ -363,7 +365,7 @@ gboolean
 wpa_flush_to_file (const char *config_file)
 {
 	GIOChannel *channel;
-	GError **error = NULL;
+	GError *error = NULL;
 	gpointer key, value, ssid, security;
 	GHashTableIter iter, iter_security;
 	gchar *out_line;
@@ -387,25 +389,27 @@ wpa_flush_to_file (const char *config_file)
 	g_io_channel_write_chars (channel,
 				  "#Generated by NetworkManager\n"
 				  "###### Global Configuration ######\n",
-				  -1, &bytes_written, error);
+				  -1, &bytes_written, &error);
+	if (error)
+		goto done;
 
 	/* Writing global information */
 	while (g_hash_table_iter_next (&iter, &key, &value)) {
 		out_line =
 		    g_strdup_printf ("%s=%s\n", (gchar *) key, (gchar *) value);
 		g_io_channel_write_chars (channel, out_line, -1, &bytes_written,
-					  error);
-		if (bytes_written == 0 || (error && *error))
+					  &error);
+		if (bytes_written == 0 || error)
 			break;
 		g_free (out_line);
 	}
-	if (error && *error) {
-		nm_log_warn (LOGD_SETTINGS, "Found error: %s", (*error)->message);
+	if (error)
 		goto done;
-	}
 	g_io_channel_write_chars (channel,
 				  "\n###### Security Configuration ######\n",
-				  -1, &bytes_written, error);
+				  -1, &bytes_written, &error);
+	if (error)
+		goto done;
 
 	g_hash_table_iter_init (&iter, wsec_table);
 	/* Writing security */
@@ -413,35 +417,34 @@ wpa_flush_to_file (const char *config_file)
 		g_hash_table_iter_init (&iter_security,
 					(GHashTable *) security);
 		g_io_channel_write_chars (channel, "network={\n", -1,
-					  &bytes_written, error);
+					  &bytes_written, &error);
+		if (error)
+			goto done;
 		while (g_hash_table_iter_next (&iter_security, &key, &value)) {
 			out_line =
 			    g_strdup_printf (need_quote ((gchar *) key) ?
 					     "\t%s=\"%s\"\n" : "\t%s=%s\n",
 					     (gchar *) key, (gchar *) value);
 			g_io_channel_write_chars (channel, out_line, -1,
-						  &bytes_written, error);
-			if (bytes_written == 0 || (error && *error))
-				break;
+						  &bytes_written, &error);
+			if (bytes_written == 0 || error)
+				goto done;
 			g_free (out_line);
 		}
-		g_io_channel_write_chars (channel,
-					  "}\n\n", -1, &bytes_written, error);
+		g_io_channel_write_chars (channel, "}\n\n", -1, &bytes_written, &error);
 
 	}
-	if (error && *error) {
-		nm_log_warn (LOGD_SETTINGS, "Found error: %s", (*error)->message);
+	g_io_channel_flush (channel, &error);
+	if (error)
 		goto done;
-	}
-	g_io_channel_flush (channel, error);
 
-	if (error && *error) {
-		nm_log_warn (LOGD_SETTINGS, "Found error: %s", (*error)->message);
-		goto done;
-	}
 	wpa_parser_data_changed = FALSE;
 	result = TRUE;
 done:
+	if (error) {
+		nm_log_warn (LOGD_SETTINGS, "Error writing WPA configuration: %s", error->message);
+		g_error_free (error);
+	}
 	g_io_channel_shutdown (channel, FALSE, NULL);
 	g_io_channel_unref (channel);
 	return result;
